@@ -34,7 +34,7 @@ from random import randint
 from micropython import const
 from .matcher import MQTTMatcher
 
-__version__ = "5.5.1"
+__version__ = "5.5.2"
 __repo__ = "https://github.com/adafruit/Adafruit_CircuitPython_MiniMQTT.git"
 
 # Client-specific variables
@@ -834,7 +834,8 @@ class MQTT:
                 feed = subscribed_topics.pop()
                 self.subscribe(feed)
 
-    def loop(self, timeout=1):
+    def loop(self, timeout=0):
+        # pylint: disable = too-many-return-statements
         """Non-blocking message loop. Use this method to
         check incoming subscription messages.
         Returns response codes of any messages received.
@@ -842,6 +843,7 @@ class MQTT:
         :param int timeout: Socket timeout, in seconds.
 
         """
+
         if self._timestamp == 0:
             self._timestamp = time.monotonic()
         current_time = time.monotonic()
@@ -854,11 +856,28 @@ class MQTT:
                 )
             rcs = self.ping()
             return rcs
+
+        stamp = time.monotonic()
         self._sock.settimeout(timeout)
-        rc = self._wait_for_msg()
-        return [rc] if rc else None
+        rcs = []
+
+        while True:
+            rc = self._wait_for_msg(timeout)
+            if rc is None:
+                break
+            if time.monotonic() - stamp > self._recv_timeout:
+                if self.logger is not None:
+                    self.logger.debug(
+                        f"Loop timed out, message queue not empty after {self._recv_timeout}s"
+                    )
+                break
+            rcs.append(rc)
+
+        return rcs if rcs else None
 
     def _wait_for_msg(self, timeout=0.1):
+        # pylint: disable = too-many-return-statements
+
         """Reads and processes network events."""
         # CPython socket module contains a timeout attribute
         if hasattr(self._socket_pool, "timeout"):
